package migl.lisp;

import java.math.BigInteger;
import java.util.Stack;

import migl.util.ConsList;
import migl.util.ConsListFactory;

public class LispImpl implements Lisp {

	@Override
	public Object parse(String expr) throws LispError {
		/*
		 * Implementer Grammaire
		 * X = E | F
		 * E = bool | entier | decimal | string
		 * F = '('X*')'
		 */
		Stack<String> explode = this.explode(expr);
		while(explode.size() > 0) {
			String val = explode.remove(0);
			//Analyse le type d'expression
			if(val.equals("(")){
				return this.parseList(explode);
			} else {
				//Element seul
				if(explode.size() > 1) {
					throw new LispError("Invalid format");
				}
				return this.parseElement(val);
			}
		}
		return null;
	}

	@Override
	public Object evaluate(Object ex) throws LispError {
		// TODO Auto-generated method stub
		return null;
	}
	
	/**
	 * Découpe un string pour l'analyse Lisp
	 * @param expr L'expression à analyser
	 * @return L'expression découper
	 */
	private Stack<String> explode(String expr) {
		String[] tmp = expr.split("[ |\t]");
		Stack<String> result = new Stack<>();
		for(String str : tmp) {
			if(str.length() > 0) {
				if(str.charAt(0) == '(') {
					result.add("" + str.charAt(0));
					if(str.charAt(str.length() - 1) == ')') {
						result.add(str.substring(1, str.length() - 1));
						result.add("" + str.charAt(str.length() - 1));
					} else {
						result.add(str.substring(1));
					}
				} else if(str.charAt(str.length() - 1) == ')'){
					result.add(str.substring(0, str.length() - 2));
					result.add("" + str.charAt(str.length() - 1));
				} else {
					result.add(str);
				}
			}
		}
		return result;
	}
	
	/**
	 * Parse un element depuis une chaine de caractere
	 * @param val La chaine de caractere
	 * @return L'objet parser
	 */
	private Object parseElement(String val) {
		try {
			BigInteger bi = new BigInteger(val);
			return bi;
		} catch(NumberFormatException ex) {
			//Rien
		}
		try {
			Double d = Double.parseDouble(val);
			return d;
		} catch(NumberFormatException ex) {
			//Rien
		}
		try {
			LispBoolean lbool = LispBoolean.valueOf(val);
			return lbool;
		} catch(IllegalArgumentException ex) {
			//Rien
		}
		return val;
	}
	
	private Object parseList(Stack<String> pile) throws LispError {
		ConsList<Object> list = ConsListFactory.nil();
		String val = pile.remove(0);
		while(pile.size() > 0 && !val.equals(")")) {
			if(val.equals("(")) {
				list = list.append(this.parseList(pile));
			} else {
				list = list.append(this.parseElement(val));
			}
			val = pile.remove(0);
		}
		//Si arret cause pile vide erreur
		if(!val.equals(")")) {
			throw new LispError("Invalid Format");
		}
		return list;
	}

}
